/*
----------------------------------------------------------------------------
' Adapted for VB.NET from VB6 sample code provided by:
' Copyright 1996-2008 VBnet, Randy Birch. All Rights Reserved Worldwide.
'        Terms of use http://vbnet.mvps.org/terms/pages/terms.htm
'----------------------------------------------------------------------------

'Original code was for VB6.  Data types are different in VB.NET so function
'prototypes had to match both x86 and x64 build
*/

using System;
using System.Runtime.InteropServices;

namespace ClientEx
{
    class BrowseFolderDlg
    {
        public enum CSIDL {
            CSIDL_ADMINTOOLS        = 0x0030,
            CSIDL_ALTSTARTUP        = 0x001d,
            CSIDL_APPDATA           = 0x001a,
            CSIDL_BITBUCKET         = 0x000a,
            CSIDL_CDBURN_AREA           = 0x003b,
            CSIDL_COMMON_ADMINTOOLS     = 0x002f,
            CSIDL_COMMON_ALTSTARTUP     = 0x001e,
            CSIDL_COMMON_APPDATA        = 0x0023,
            CSIDL_COMMON_DESKTOPDIRECTORY   = 0x0019,
            CSIDL_COMMON_DOCUMENTS      = 0x002e,
            CSIDL_COMMON_FAVORITES      = 0x001f,
            CSIDL_COMMON_MUSIC          = 0x0035,
            CSIDL_COMMON_OEM_LINKS      = 0x003a,
            CSIDL_COMMON_PICTURES       = 0x0036,
            CSIDL_COMMON_PROGRAMS       = 0X0017,
            CSIDL_COMMON_STARTMENU      = 0x0016,
            CSIDL_COMMON_STARTUP        = 0x0018,
            CSIDL_COMMON_TEMPLATES      = 0x002d,
            CSIDL_COMMON_VIDEO          = 0x0037,
            CSIDL_COMPUTERSNEARME       = 0x003d,
            CSIDL_CONNECTIONS           = 0x0031,
            CSIDL_CONTROLS          = 0x0003,
            CSIDL_COOKIES           = 0x0021,
            CSIDL_DESKTOP           = 0x0000,
            CSIDL_DESKTOPDIRECTORY      = 0x0010,
            CSIDL_DRIVES            = 0x0011,
            CSIDL_FAVORITES         = 0x0006,
            CSIDL_FLAG_CREATE           = 0x8000,
            CSIDL_FLAG_DONT_VERIFY      = 0x4000,
            CSIDL_FLAG_MASK         = 0xFF00,
            CSIDL_FLAG_NO_ALIAS         = 0x1000,
            CSIDL_FLAG_PER_USER_INIT    = 0x0800,
            CSIDL_FONTS             = 0x0014,
            CSIDL_HISTORY           = 0x0022,
            CSIDL_INTERNET          = 0x0001,
            CSIDL_INTERNET_CACHE        = 0x0020,
            CSIDL_LOCAL_APPDATA         = 0x001c,
            CSIDL_MYDOCUMENTS           = 0x000c,
            CSIDL_MYMUSIC           = 0x000d,
            CSIDL_MYPICTURES        = 0x0027,
            CSIDL_MYVIDEO           = 0x000e,
            CSIDL_NETHOOD           = 0x0013,
            CSIDL_NETWORK           = 0x0012,
            CSIDL_PERSONAL          = 0x0005,
            CSIDL_PRINTERS          = 0x0004,
            CSIDL_PRINTHOOD         = 0x001b,
            CSIDL_PROFILE           = 0x0028,
            CSIDL_PROGRAM_FILES         = 0x0026,
            CSIDL_PROGRAM_FILES_COMMON      = 0x002b,
            CSIDL_PROGRAM_FILES_COMMONX86   = 0x002c,
            CSIDL_PROGRAM_FILESX86      = 0x002a,
            CSIDL_PROGRAMS          = 0x0002,
            CSIDL_RECENT            = 0x0008,
            CSIDL_RESOURCES         = 0x0038,
            CSIDL_RESOURCES_LOCALIZED       = 0x0039,
            CSIDL_SENDTO            = 0x0009,
            CSIDL_STARTUP           = 0x0007,
            CSIDL_SYSTEM            = 0x0025,
            CSIDL_SYSTEMX86         = 0x0029,
            CSIDL_TEMPLATES         = 0x0015,
            CSIDL_WINDOWS           = 0x0024   
        };

       // Constants for sending and receiving messages in BrowseCallBackProc
        public const int WM_USER = 0x400;
        public const int BFFM_INITIALIZED = 1;
        public const int BFFM_SELCHANGED = 2;
        public const int BFFM_VALIDATEFAILEDA = 3;
        public const int BFFM_VALIDATEFAILEDW = 4;
        public const int BFFM_IUNKNOWN = 5; // provides IUnknown to client. lParam: IUnknown*
        public const int BFFM_SETSTATUSTEXTA = WM_USER + 100;
        public const int BFFM_ENABLEOK = WM_USER + 101;
        public const int BFFM_SETSELECTIONA = WM_USER + 102;
        public const int BFFM_SETSELECTIONW = WM_USER + 103;
        public const int BFFM_SETSTATUSTEXTW = WM_USER + 104;
        public const int BFFM_SETOKTEXT = WM_USER + 105; // Unicode only
        public const int BFFM_SETEXPANDED = WM_USER + 106; // Unicode only

        // Browsing for directory.
        private const uint BIF_RETURNONLYFSDIRS   = 0x0001;  // For finding a folder to start document searching
        private const uint BIF_DONTGOBELOWDOMAIN  = 0x0002;  // For starting the Find Computer
        private const uint BIF_STATUSTEXT     = 0x0004;  // Top of the dialog has 2 lines of text for BROWSEINFO.lpszTitle and one line if
        // this flag is set.  Passing the message BFFM_SETSTATUSTEXTA to the hwnd can set the
        // rest of the text.  This is not used with BIF_USENEWUI and BROWSEINFO.lpszTitle gets
        // all three lines of text.
        private const uint BIF_RETURNFSANCESTORS  = 0x0008;
        private const uint BIF_EDITBOX        = 0x0010;   // Add an editbox to the dialog
        private const uint BIF_VALIDATE       = 0x0020;   // insist on valid result (or CANCEL)

        private const uint BIF_NEWDIALOGSTYLE     = 0x0040;   // Use the new dialog layout with the ability to resize
        // Caller needs to call OleInitialize() before using this API
        private const uint BIF_USENEWUI  = 0x0040 + 0x0010; //(BIF_NEWDIALOGSTYLE | BIF_EDITBOX);

        private const uint BIF_BROWSEINCLUDEURLS  = 0x0080;   // Allow URLs to be displayed or entered. (Requires BIF_USENEWUI)
        private const uint BIF_UAHINT         = 0x0100;   // Add a UA hint to the dialog, in place of the edit box. May not be combined with BIF_EDITBOX
        private const uint BIF_NONEWFOLDERBUTTON  = 0x0200;   // Do not add the "New Folder" button to the dialog.  Only applicable with BIF_NEWDIALOGSTYLE.
        private const uint BIF_NOTRANSLATETARGETS = 0x0400;  // don't traverse target as shortcut

        private const uint BIF_BROWSEFORCOMPUTER  = 0x1000;  // Browsing for Computers.
        private const uint BIF_BROWSEFORPRINTER   = 0x2000;// Browsing for Printers
        private const uint BIF_BROWSEINCLUDEFILES = 0x4000; // Browsing for Everything
        private const uint BIF_SHAREABLE      = 0x8000;  // sharable resources displayed (remote shares, requires BIF_USENEWUI)

//        public delegate int BrowseCallBackProc(IntPtr hwnd, int msg, IntPtr lp, IntPtr wp);       
        public delegate Int32 BrowseCallbackProc(IntPtr hwnd, UInt32 uMsg, Int32 lParam, Int32 lpData);

        [StructLayout(LayoutKind.Sequential)]
        public struct BROWSEINFO
        {
            public IntPtr hwndOwner;                // Handle to the owner window for the dialog box.
            public IntPtr pidlRoot;                 // Pointer to an item identifier list (PIDL) specifying the location of
                                                    // the root folder from which to start
                                                    // browsing.
            [MarshalAs(UnmanagedType.LPStr)]        // Address of a buffer to receive the display name of the 
            public String pszDisplayName;           // folder selected by the user.
            [MarshalAs(UnmanagedType.LPStr)]        // Address of a null-terminated string that is displayed 
            public String lpszTitle;                // above the tree view control in the dialog box.
            public UInt32 ulFlags;                  // Flags specifying the options for the dialog box.
            [MarshalAs(UnmanagedType.FunctionPtr)]  // Address of an application-defined function that the 
            public BrowseCallbackProc lpfn;         // dialog box calls when an event occurs.
            public Int32 lParam;                    // Application-defined value that the dialog box passes to the callback function
            public Int32 iImage;                    // Variable to receive the image associated with the selected folder.
        };

        // Displays a dialog box that enables the user to select a Shell folder. 
        [DllImport("shell32.dll")]
        public static extern IntPtr SHBrowseForFolder(
            ref BROWSEINFO lbpi);    // Pointer to a BROWSEINFO structure that contains
        // information used to display 
        // the dialog box. 

        /* 
         HRESULT SHGetSpecialFolderLocation(
          __in   HWND hwndOwner,
          __in   int nFolder,
          __out  PIDLIST_ABSOLUTE *ppidl
        );
         */

        // Note:  HResult is a long which is the same 32-bit size on x86 and x64
        [DllImport("shell32.dll", CharSet=CharSet.Ansi)]
        public static extern UInt32 SHGetSpecialFolderLocation(IntPtr hWnd, 
            CSIDL nFolder, 
            ref IntPtr pidl);

        //Private Declare Sub CoTaskMemFree Lib "ole32.dll" (ByVal pv As IntPtr)        
        [DllImport("ole32.dll")]
        static extern IntPtr CoTaskMemAlloc(uint cb);
            
        [DllImport("ole32.dll")]
        static extern void CoTaskMemFree(IntPtr pv);
        
        public static string GetBrowseNetworkWorkstation(IntPtr hwndOwner,
            bool bNewDialog,
            bool bNoNewFolder,
            string Title)
        {

            string Machine = "";

            try
            {
                // returns only a valid network server or
                // workstation (does not display the shares)
                BROWSEINFO bi = new BROWSEINFO();
                IntPtr pidl = System.IntPtr.Zero;
                uint iError = 0;

                iError = SHGetSpecialFolderLocation(hwndOwner,
                                                      CSIDL.CSIDL_NETWORK,
                                                      ref pidl);

                if (iError == 0)
                {

                    // fill in the required members
                    bi.hwndOwner = hwndOwner;
                    bi.pidlRoot = pidl;
                    bi.pszDisplayName = new String(' ', 256);
                    bi.lpszTitle = Title;
                    bi.ulFlags = BIF_BROWSEFORCOMPUTER;

                    //This does not work on w2003 & XP
                    //If bNewDialog Then
                    //    bi.ulFlags = bi.ulFlags Or BIF_NEWDIALOGSTYLE
                    //End If
                    //If bNoNewFolder Then
                    //    bi.ulFlags = bi.ulFlags Or BIF_NONEWFOLDERBUTTON
                    //End If

                    pidl = SHBrowseForFolder(ref bi);

                    if (pidl != null)
                    {

                        /* a machine is selected. Although a valid pidl
                        is returned, SHGetPathFromIDList only return
                        paths to valid file system objects, of which
                        a networked machine is not. However, the
                        BROWSEINFO displayname member does contain
                        the selected item, which we return */
                        Machine = bi.pszDisplayName;

                        CoTaskMemFree(pidl);
                    }
                } 
            }
            catch (Exception e)
            {
                System.Console.WriteLine("Could not GetBrowseNetworkWorkstation " + e.Message);
            }

            return Machine;

        }
    }
}
