﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ClientEx.DTS;
using System.Windows.Forms.DataVisualization.Charting;


namespace ClientEx
{
    public partial class ClientExForm : Form
    {
        public ClientExForm()
        {
            InitializeComponent();
        }

        // Filter variables
        public bool useInstanceFilters = false;
        public bool useCounterFilters = false;
        public DTS.PerformanceObjectInstanceFilter instanceFilter = null;
        public DTS.PerformanceCounterTestFilter counterFilter = null;
        public string ObjectName = "";
        public string CounterName = "";

        private void btnBrowseMachine_Click(object sender, EventArgs e)
        {
            // Let the user get a list of machines
            // Need to get another machine name.
            string Machine = BrowseFolderDlg.GetBrowseNetworkWorkstation(IntPtr.Zero, true, true, "Browse for Computer Processes");
            if (Machine != null)
            {
                Machine = Machine.Trim();
                if (Machine.Length > 0)
                {
                    // Set the machine in the ComboBox
                    this.tbMachineName.Text = Machine;

                    // Make the call to the proxy to get the counters for that machine
                    PerformanceDataMgmtClient client = GetPerformanceDataMgmtClient(Machine);
                    if (client == null)
                    {
                        MessageBox.Show("Could not generate proxy to get performance data from machine " + Machine + ".", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    else
                    {
                        string[] ObjectNames = null;
                        try
                        {
                            ObjectNames = client.GetCollectionObjectNames();
                            
                        }
                        catch (Exception e2)
                        {
                            MessageBox.Show("Exception occurred while getting the collection object names.\r\n\r\n" + e2.Message, "Info", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        }

                        // Clear the list box and put these strings in the list box
                        this.lbCollectedObjects.Items.Clear();

                        if (ObjectNames == null)
                        {
                            MessageBox.Show("Could not get the collection object from the machine " + Machine + ".", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                        else
                        {
                            for (int i = 0; i < ObjectNames.Length; i++)
                                this.lbCollectedObjects.Items.Add(ObjectNames[i]);
                        }

                        // Cleanup
                        if (client != null)
                            client.Close();
                    }

                }
            }        
        }

        private static PerformanceDataMgmtClient GetPerformanceDataMgmtClient(string MachineName)
        {
            string baseAddress = Properties.Settings.Default.PerfSentryDataMgmtAddress.Replace("$MachineName", MachineName);
            PerformanceDataMgmtClient proxy = new PerformanceDataMgmtClient(
                Properties.Settings.Default.PerfSentryDataMgmtBinding,
                baseAddress);
            return proxy;
        }

        private void ClientExForm_Load(object sender, EventArgs e)
        {
            // Set the default time for the time controls
            this.EndTime.Value = DateTime.Now;
            this.StartTime.Value = DateTime.Now - new TimeSpan(0, 30, 0);         

            // Set up the chart types
            string[] names = Enum.GetNames(typeof(SeriesChartType));
            Array values = Enum.GetValues(typeof(SeriesChartType));

            // Only use these 3 chart types Line, Area, StackedArea, Bar
            for (int countElements = 0; countElements <= names.Length - 1; countElements++)
            {
                if ((SeriesChartType.Area == (SeriesChartType) values.GetValue(countElements)) ||
                    (SeriesChartType.Line == (SeriesChartType) values.GetValue(countElements)) ||
                    (SeriesChartType.Column ==(SeriesChartType) values.GetValue(countElements)) ||
                    (SeriesChartType.StackedColumn == (SeriesChartType)values.GetValue(countElements)))
                {
                    this.cbChartType.Items.Add( names[countElements].ToString() );
                }
            }

            // Select the first index
            this.cbChartType.SelectedIndex = 0;

            // Select the first duration
            this.cbRefreshDuration.SelectedIndex = 0;

            EnableControls();
        }

        public SeriesChartType GetSeriesChartType( string Name )
        {
             // Set up the chart types
            string[] names = Enum.GetNames(typeof(SeriesChartType));
            Array values = Enum.GetValues(typeof(SeriesChartType));
            for (int countElements = 0; countElements <= names.Length - 1; countElements++)
            {
                if ( names[countElements] ==  Name )
                    return (SeriesChartType) values.GetValue(countElements);
            }

            return SeriesChartType.Column;
        }

        private void lbCollectedObjects_SelectedValueChanged(object sender, EventArgs e)
        {
            System.Console.WriteLine("Collected Objects Selected Value = " + this.lbCollectedObjects.SelectedItem);

            // Get the counters for the object
            // Make sure machine is selected
            string Machine = this.tbMachineName.Text;
            if ((Machine != null) && (Machine.Length > 0))
            {
                // Get the object name
                string ObjectName = this.lbCollectedObjects.SelectedItem as string;
                if ((ObjectName != null) && (ObjectName.Length > 0))
                {
                    // Make sure object is selected
                    PerformanceDataMgmtClient client = GetPerformanceDataMgmtClient(Machine);
                    if (client == null)
                    {
                        MessageBox.Show("Could not get the performance data client interface to retrieve counters for " + ObjectName);
                    }
                    else
                    {
                        string[] CounterNames = null;
                        try
                        {
                            CounterNames = client.GetCounterNames(ObjectName);
                        }
                        catch (Exception e3)
                        {
                            System.Console.WriteLine(e3.Message);
                        }
                        finally
                        {
                            if (client != null)
                                client.Close();
                        }

                        if (CounterNames == null)
                        {
                            MessageBox.Show("Could not get counter names of the object " + ObjectName);
                        }
                        else
                        {
                            this.lbCounters.Items.Clear();
                            for (int i = 0; i < CounterNames.Length; i++)
                                this.lbCounters.Items.Add(CounterNames[i]);
                        }
                        
                    }
                }
            }

            EnableControls();
        }

        private void lbCounters_SelectedValueChanged(object sender, EventArgs e)
        {
            EnableControls();

            RefreshData();
        }
        
        
        public void RefreshData()
        {
            // Get the data for this counter from the selected object on the selected machine
            string Machine = this.tbMachineName.Text;
            if ((Machine != null) && (Machine.Length < 1))
                return;

            string Object = this.lbCollectedObjects.SelectedItem as string;
            if ((Object == null) || (Object.Length < 1))
                return;

            string Counter = this.lbCounters.SelectedItem as string;
            if ((Counter == null) || (Counter.Length < 1))
                return;

            
            // Make sure object is selected
            PerformanceDataMgmtClient client = GetPerformanceDataMgmtClient(Machine);
            if (client == null)
            {
                MessageBox.Show("Could not get the performance data client interface to retrieve data from " + Machine + " for " + Object + " " + Counter);
            }
            else
            {
                PerformanceDataRequest request = new PerformanceDataRequest();
                try
                {
                    // Set up the performance object
                    PerformanceObject[] objects = new PerformanceObject[1];
                    objects[0] = new PerformanceObject();
                    objects[0].Name = Object;

                    // Set up the performance counter
                    objects[0].Counters = new PerformanceCounter[1];
                    objects[0].Counters[0] = new PerformanceCounter();
                    objects[0].Counters[0].Name = Counter;

                    request.ObjectsRequested = objects;

                    // Update the start & end time with the auto refresh
                    if (this.cbAutoRefresh.Checked == true)
                    {
                        if (this.cbRefreshDuration.SelectedItem==null)
                        {
                            MessageBox.Show("Error", "You must select a refresh duration when auto refresh is selected.", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                            return;
                        }

                        int RefreshDuration = 0;
                        try
                        {
                            RefreshDuration = int.Parse(cbRefreshDuration.Text);
                        }
                        catch (Exception e)
                        {
                            MessageBox.Show("Specify an integer number for the refresh duration.","Error",MessageBoxButtons.OK, MessageBoxIcon.Stop);
                        }

                        this.EndTime.Value = DateTime.Now;
                        this.StartTime.Value = this.EndTime.Value - new TimeSpan(0, RefreshDuration, 0);
                    }
                    
                    request.StartTime = this.StartTime.Value;
                    request.EndTime = this.EndTime.Value;
                    

                    // use instance filter
                    if ((this.cbUseInstanceFilter.Checked == true) && (this.instanceFilter != null))
                    {
                        request.InstanceFilters = new PerformanceObjectInstanceFilter[1];
                        request.InstanceFilters[0] = this.instanceFilter;
                    }

                    // use counter filter
                    if (this.cbUseCounterFilter.Checked == true)
                    {
                        request.PerformanceCounterTestFilters = new PerformanceCounterTestFilter[1];
                        request.PerformanceCounterTestFilters[0] = new PerformanceCounterTestFilter();
                        request.PerformanceCounterTestFilters[0].ObjectName = Object;
                        request.PerformanceCounterTestFilters[0].CounterName = Counter;
                        switch (cbCounterFilterOperator.Text)
                        {
                            case ">":
                                request.PerformanceCounterTestFilters[0].Type = PerformanceCounterTestType.GT;
                                break;

                            case ">=":
                                request.PerformanceCounterTestFilters[0].Type = PerformanceCounterTestType.GTE;
                                break;
                            case "<=":
                                request.PerformanceCounterTestFilters[0].Type = PerformanceCounterTestType.LTE;
                                break;
                            case "<":
                                request.PerformanceCounterTestFilters[0].Type = PerformanceCounterTestType.LT;
                                break;
                            case "=":
                                request.PerformanceCounterTestFilters[0].Type = PerformanceCounterTestType.EQ;
                                break;
                            default:
                                MessageBox.Show("Select counter filter type when the use counter checkbox is checked.", "Error",MessageBoxButtons.OK, MessageBoxIcon.Error);
                                break;
                        }
                        
                        try
                        {
                            request.PerformanceCounterTestFilters[0].ThresholdValue = double.Parse(tbCounterFilterValue.Text);
                        }
                        catch (Exception e)
                        {
                            System.Console.WriteLine(e.Message);
                            MessageBox.Show("Counter filter value must be numeric.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            return;
                        }
                    }

                    ArrayList dataSamples = new ArrayList();

                    // Clear out the items
                    this.lvValues.Items.Clear();
                    this.DataChart.Titles.Clear();
                        
                    PerformanceData data = client.GetPerformanceData(request);
                    if (data != null)
                    {
                        // Data is returned in the ObjectData array for each object name
                        for (int i = 0; i < data.ObjectNames.Length; i++)
                        {
                            // If there are no samples of the object, then it should not be returned
                            if (data.ObjectData[i] == null)
                                continue;

                            // samples of the object are stored in the ObjectData array by object and sorted by Time
                            for (int s = 0; s < data.ObjectData[i].Length; s++)
                            {
                                // Get the sth sample for the ith object
                                PerformanceObjectData sample = data.ObjectData[i][s];

                                // Each object has a number of instances, for each instance output the counter values
                                for (int j = 0; j < sample.InstanceNames.Length; j++)
                                {
                                    // Each instance has a number of values ordered by the request counter order
                                    for (int k = 0; k < sample.Values[j].Length; k++)
                                    {
                                        if (sample.Values[j][k] != null)
                                        {
                                            // Internal Representation of the data redundant memory usage but illustrates concept
                                            CounterDataSample dataSample = new CounterDataSample(sample.InstanceNames[j],
                                                sample.CollectionTime,
                                                Object,
                                                Counter,
                                                (double)sample.Values[j][k]);
                                            dataSamples.Add(dataSample);

                                        }
                                    }
                                }
                            }
                        }

                        // Adds Titles if there is data returned
                        if (dataSamples.Count != 0)
                        {
                            this.DataChart.Titles.Add(Object + "/" + Counter + " on " + Machine);
                            this.DataChart.Titles.Add("From " + StartTime.Value + " to " + EndTime.Value);
                        }

                        // Add the new items & data to the chart
                        IEnumerator e = this.DataChart.Series.GetEnumerator();
                        while (e.MoveNext())
                        {
                            Series s = (Series) e.Current;
                            s.Points.Clear();
                        }

                        this.DataChart.Series.Clear();

                        Hashtable indexedInstances = new Hashtable();

                        // There is one data sample for each instance at a particular time
                        for (int i = 0; i < dataSamples.Count; i++)
                        {
                            CounterDataSample sample = dataSamples[i] as CounterDataSample;

                            // Define the list items
                            ListViewItem lvi = new ListViewItem(sample.CollectionTime.ToString());
                            lvi.SubItems.Add(sample.Instance);
                            lvi.SubItems.Add(sample.Value.ToString());

                            // Add the list items to the ListView
                            this.lvValues.Items.Add(lvi);

                            // Add one series to the chart for each instance of the object requested
                            Series s = null;

                            // use 'uninstanced' for the instance name
                            string InstanceName = "";
                            if (sample.Instance.Length == 0)
                                InstanceName = "Uninstanced";
                            else
                                InstanceName = sample.Instance;

                            s = this.DataChart.Series.FindByName(InstanceName);                            
                            if (s == null)
                            {
                                s = new System.Windows.Forms.DataVisualization.Charting.Series(InstanceName);                          
                                this.DataChart.Series.Add(s);
                            }

                            // Set the ChartType
                            if (cbChartType.SelectedItem == null)
                            {
                                s.ChartType = SeriesChartType.Column;
                            }
                            else
                            {
                                s.ChartType = GetSeriesChartType(cbChartType.SelectedItem.ToString());
                            }

                            // Add the point CollectionTime,Value
                            if ((s != null) && (s.Points != null))
                            {
                                string CollectTime = sample.CollectionTime.ToShortDateString() + " " + sample.CollectionTime.ToShortTimeString();
                                s.Points.AddXY(CollectTime, sample.Value);

                            }
                        }

                        if (this.lvValues.Items.Count == 0)
                        {
                            MessageBox.Show("No data was found for the specified time period.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        }
                    }
                }
                catch (Exception e3)
                {
                    MessageBox.Show("Could not get performance counter data " + e3.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);

                    System.Console.WriteLine(e3.Message);
                }
                finally
                {
                    if (client != null)
                        client.Close();
                }
            }
        }

        private void btnRefresh_Click(object sender, EventArgs e)
        {
            RefreshData();
        }

        private void btnUseCurrentEndTime_Click(object sender, EventArgs e)
        {
            this.EndTime.Value = System.DateTime.Now;
        }

        private void btnUseCurrentStartTime_Click(object sender, EventArgs e)
        {
            this.StartTime.Value = System.DateTime.Now;
        }

        private void cbChartType_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cbChartType.SelectedItem == null)
                return;
            if (this.DataChart.Series == null)
                return;

            // Add the new items & data to the chart
            IEnumerator e2 = this.DataChart.Series.GetEnumerator();
            while (e2.MoveNext())
            {
                Series s = (Series)e2.Current;
                s.ChartType = GetSeriesChartType(cbChartType.SelectedItem.ToString());
            }
        }

        private void timerAutoRefresh_Tick(object sender, EventArgs e)
        {
            if (this.cbAutoRefresh.Checked)
            {
                RefreshData();
            }
        }

        private void EnableControls()
        {
            this.btnFilters.Enabled = this.cbUseInstanceFilter.Checked 
                && (this.lbCollectedObjects.SelectedItem != null);

            this.cbRefreshDuration.Enabled = this.cbAutoRefresh.Checked;
            this.StartTime.Enabled = !this.cbAutoRefresh.Checked;
            this.EndTime.Enabled = !this.cbAutoRefresh.Checked;
            this.btnUseCurrentEndTime.Enabled = !this.cbAutoRefresh.Checked;
        }

        private void btnFilters_Click(object sender, EventArgs e)
        {
            // Make sure the filter button is enabled for this
            if (this.lbCollectedObjects.SelectedItem == null)
                return;

            // launch the filter dialog
            FilterDialog f = new FilterDialog();
            f.instanceFilter = this.instanceFilter;
            f.CounterName = this.lbCounters.SelectedItem as string;
            f.ObjectName = this.lbCollectedObjects.SelectedItem as string;            
            
            DialogResult result = f.ShowDialog();
            if (result == DialogResult.OK)
            {
                this.instanceFilter = f.instanceFilter;
                if (this.cbUseInstanceFilter.Checked)
                    RefreshData();
            }
        }

        private void cbChartType_SelectedIndexChanged_1(object sender, EventArgs e)
        {
            RefreshData();
        }
        
        private void cbUseCounterFilter_CheckedChanged(object sender, EventArgs e)
        {
            RefreshData();
        }

        private void cbUseInstanceFilter_CheckedChanged(object sender, EventArgs e)
        {
            EnableControls();

            RefreshData();
        }

        private void cbAutoRefresh_CheckedChanged_1(object sender, EventArgs e)
        {
            EnableControls();

            this.timerAutoRefresh.Enabled = this.cbAutoRefresh.Checked;

            if (this.cbAutoRefresh.Checked)
            {
                RefreshData();
            }
        }

        private void btnRefresh_Click_1(object sender, EventArgs e)
        {
            RefreshData();
        }

        private void cbRefreshDuration_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (this.cbAutoRefresh.Checked)
            {
                RefreshData();
            }
        }

    }
}
