﻿using System;
using System.Collections;
using CollectAll.DTS;

namespace CollectAll
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                if (args.Length != 1)
                    return;

                // Get the object name
                string ObjectName = args[0];

                PerformanceDataMgmtClient client = new PerformanceDataMgmtClient("BasicHttpBinding_IPerformanceDataMgmt");

                // Set up the object we would like to collect
                DTS.PerformanceObject perfObject = new DTS.PerformanceObject();
                perfObject.Name = ObjectName;

                string[] CounterNames = client.GetCounterNames(ObjectName);
                if ((CounterNames == null) || (CounterNames.Length == 0))
                {
                    System.Console.WriteLine("Could not find counters being collected for " + ObjectName);
                    return;
                }

                // Create the performance counters to receive
                perfObject.Counters = new DTS.PerformanceCounter[CounterNames.Length];                
                for (int i = 0; i < CounterNames.Length; i++)
                {
                    DTS.PerformanceCounter counter = new DTS.PerformanceCounter();
                    counter.Name = CounterNames[i];
                    perfObject.Counters[i] = counter;
                }

                // Object list
                ArrayList ObjectList = new ArrayList();
                ObjectList.Add(perfObject);
                
                // Add the object to the list of objects for the request
                PerformanceObject[] perfObjects = new PerformanceObject[ObjectList.Count];
                for (int i = 0; i < ObjectList.Count; i++)
                   perfObjects[i] = (PerformanceObject) ObjectList[i];
                
                // Set up the request
                DTS.PerformanceDataRequest request = new DTS.PerformanceDataRequest();
                request.EndTime = DateTime.Now;
                request.StartTime = request.EndTime - new TimeSpan(0, 30, 0);
                request.ObjectsRequested = perfObjects;

                // Get the performance data
                DTS.PerformanceData data = client.GetPerformanceData(request);
                if (data==null)
                    throw new Exception("Could not get performance data.");

                System.Console.WriteLine("PerformanceData from " + data.StartTime + " to " + data.EndTime );

                // Data is returned in the ObjectData array for each object name
                for (int i = 0; i < data.ObjectNames.Length; i++)                
                {
                    // If there are no samples of the object, then it should not be returned
                    if (data.ObjectData[i] == null)
                        continue;

                    // samples of the object are stored in the ObjectData array by object and sorted by Time
                    for (int s=0; s < data.ObjectData[i].Length; s++)
                    {                     
                        // Get the sth sample for the ith object
                        PerformanceObjectData sample = data.ObjectData[i][s];
                                                
                        string message = data.ObjectNames[i] + "," + sample.CollectionTime.ToString();

                        // Each object has a number of instances, for each instance output the counter values
                        for (int j=0; j < sample.InstanceNames.Length; j++)
                        {
                            message =  message + "\r\n" + sample.InstanceNames[j];

                            // Each instance has a number of values ordered by the request counter order
                            for (int k = 0; k < sample.Values[j].Length; k++)
                            {                            
                                message = message + ",";
                                
                                if (sample.Values[j][k] != null)
                                    message += sample.Values[j][k];
                            }
                        }

                        System.Diagnostics.Trace.WriteLine(message);
                        System.Console.WriteLine(message);
                    }
                }

            }
            catch (Exception e)
            {
                System.Diagnostics.Trace.WriteLine(e.Message);
                System.Console.WriteLine(e.Message);
                if (e.InnerException != null)
                {
                    System.Diagnostics.Trace.WriteLine(e.InnerException);
                    System.Console.WriteLine(e.InnerException);
                }
            }
        }
    }
}
