﻿using System;
using System.Collections;
using CollectEx.DTS;

namespace CollectEx
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                PerformanceDataMgmtClient client = new PerformanceDataMgmtClient("BasicHttpBinding_IPerformanceDataMgmt");
                DTS.PerformanceCounter ProcessorTime = new DTS.PerformanceCounter();
                ProcessorTime.Name = "% Processor Time";
                DTS.PerformanceCounter UserTime = new DTS.PerformanceCounter();
                UserTime.Name = "% User Time";
                DTS.PerformanceCounter IdProcess = new DTS.PerformanceCounter();
                IdProcess.Name = "ID Process";

                // Set up the object we would like to collect
                DTS.PerformanceObject perfObject = new DTS.PerformanceObject();
                perfObject.Name = "Process";

                perfObject.Counters = new PerformanceCounter[3];
                perfObject.Counters[0] = ProcessorTime;
                perfObject.Counters[1] = UserTime;
                perfObject.Counters[2] = IdProcess;
                
                // Object list
                ArrayList ObjectList = new ArrayList();
                ObjectList.Add(perfObject);
                
                // Get the counters for the thread object and retrieve them all
                string[] CounterNames = client.GetCounterNames("Memory");
                if (CounterNames != null)
                {
                    // Get the thread object
                    DTS.PerformanceObject memoryObject = new DTS.PerformanceObject();
                    memoryObject.Name = "Memory";
                    memoryObject.Counters = new PerformanceCounter[CounterNames.Length];
                    for (int i = 0; i < CounterNames.Length; i++)
                    {
                        PerformanceCounter pc = new PerformanceCounter();
                        pc.Name = CounterNames[i];
                        memoryObject.Counters[i] = pc;
                    }

                    ObjectList.Add(memoryObject);
                }

                // Add the object to the list of objects for the request
                PerformanceObject[] perfObjects = new PerformanceObject[ObjectList.Count];
                for (int i = 0; i < ObjectList.Count; i++)
                   perfObjects[i] = (PerformanceObject) ObjectList[i];
                
                // Set up the filter to return only the svchost dmperfss instances of the process object
                DTS.PerformanceObjectInstanceFilter filter = new DTS.PerformanceObjectInstanceFilter();
                filter.ObjectName = "Process";
                filter.InstanceNames = new string[] { "svchost", "DmPerfss" };

                // Set up a counter threshold filter for % Processor Time, instances which don't meet the test will be not returned
                DTS.PerformanceCounterTestFilter counterThreshold = new DTS.PerformanceCounterTestFilter();
                counterThreshold.ObjectName = "Process";
                counterThreshold.CounterName = "% Processor Time";
                counterThreshold.ThresholdValue = 0.1;
                counterThreshold.Type = DTS.PerformanceCounterTestType.GT;
                              
                // Set up the request
                DTS.PerformanceDataRequest request = new DTS.PerformanceDataRequest();
                request.EndTime = DateTime.Now;
                request.StartTime = request.EndTime - new TimeSpan(0, 30, 0);
                request.ObjectsRequested = perfObjects;
                request.InstanceFilters = new DTS.PerformanceObjectInstanceFilter[] { filter };
                request.PerformanceCounterTestFilters = new DTS.PerformanceCounterTestFilter[] { counterThreshold };

                // Get the performance data
                DTS.PerformanceData data = client.GetPerformanceData(request);
                if (data==null)
                    throw new Exception("Could not get performance data.");

                System.Console.WriteLine("PerformanceData from " + data.StartTime + " to " + data.EndTime );

                // Data is returned in the ObjectData array for each object name
                for (int i = 0; i < data.ObjectNames.Length; i++)                
                {
                    // If there are no samples of the object, then it should not be returned
                    if (data.ObjectData[i] == null)
                        continue;

                    // samples of the object are stored in the ObjectData array by object and sorted by Time
                    for (int s=0; s < data.ObjectData[i].Length; s++)
                    {
                     
                        // Get the sth sample for the ith object
                        PerformanceObjectData sample = data.ObjectData[i][s];
                                                
                        string message = data.ObjectNames[i] + "," + sample.CollectionTime.ToString();

                        // Each object has a number of instances, for each instance output the counter values
                        for (int j=0; j < sample.InstanceNames.Length; j++)
                        {
                            message =  message + "\r\n" + sample.InstanceNames[j];

                            // Each instance has a number of values ordered by the request counter order
                            for (int k = 0; k < sample.Values[j].Length; k++)
                            {                            
                                message = message + ",";

                                if (sample.Values[j][k] != null)
                                    message += sample.Values[j][k];
                            }
                        }

                        System.Diagnostics.Trace.WriteLine(message);
                        System.Console.WriteLine(message);
                    }
                }

            }
            catch (Exception e)
            {
                System.Diagnostics.Trace.WriteLine(e.Message);
                System.Console.WriteLine(e.Message);
                if (e.InnerException != null)
                {
                    System.Diagnostics.Trace.WriteLine(e.InnerException);
                    System.Console.WriteLine(e.InnerException);
                }
            }
        }
    }
}
