# example collect script to collect data from the collection service proxy by specifying all counters

# execute this before the script so that the DTS proxy variable is set up:

# $proxy = New-WebServiceProxy -Uri 'http://localhost:5557/DTSPSP' -namespace DTS

# this can only be done once per powershell session since powershell has a bug
# that will create a superfluous casting error if the proxy is created twice

function collectall
{
param([System.String] $objectname)

if ($objectname -eq '')
{
  Write-Host 'Please specify an object name.'
  return
}

# create object to collect
$object = New-Object DTS.PerformanceObject
$object.Name = $objectname

# get all the counters
$counternames = $proxy.GetCounterNames($objectname)
if ($counternames.Length -eq 0)
{
  Write-Host 'There were no counters found being collected for '$objectname'.'
  return
}

# set up the object counter array
$counters = @()
for ($c = 0; $c -lt $counternames.Length; $c++)
{
  $counter = New-Object DTS.PerformanceCounter
  $counter.Name = $counternames[$c]
  $counters += $counter
}
$object.Counters = $counters

# set up the time
$endtime = [System.DateTime]::Now
$duration = New-Object System.TimeSpan(0,0,30,0)
$starttime = $endtime - $duration

# create the request
$request = New-Object DTS.PerformanceDataRequest
$request.ObjectsRequested = @($object)
$request.StartTime = $starttime
$request.EndTime = $endtime
$request.StartTimeSpecified = $true
$request.EndTimeSpecified = $true

Write-Host 'Collecting from ' $uri ' Object(s) ' $request.ObjectsRequested ' FilteredInstances(s) ' $request.InstanceFilters
Write-Host 'From ' $starttime ' To ' $endtime

# get the data
$data = $proxy.GetPerformanceData($request)

# Data is returned in the ObjectData array for each object name
for ($i = 0; $i -lt $data.ObjectNames.Length; $i++)                
{
    # samples of the object are stored in the ObjectData array sorted by Time
    for ($s = 0; $s -lt $data.ObjectData[$i].Length; $s++)
    {
        # Get the sample of the ith object
        $sample = $data.ObjectData[$i][$s]
                                
        $message = $data.ObjectNames[$i] + ',' + $sample.CollectionTime
               
        # Each sample has a number of instances, for each instance output the counter values
        for ($j=0; $j -lt $sample.InstanceNames.Length; $j++)        
        {
        
            $message =  $message + "`r`n" + $sample.InstanceNames[$j]

            # Each instance has a number of values ordered by the request counter order
            for ($k = 0; $k -lt $sample.Values[$j].Length; $k++)
            {                            
                $message = $message + ','

		# uncomment line below to see counter names in the data
		# $message = $message + $counternames[$k] + ","

                if ($sample.Values[$j][$k] -ne $null)
                {
                    $message = $message + $sample.Values[$j][$k]
                }
            }
        }

       Write-Host $message
    }
}

$uri = $null
$message = $null
$object = $null
$counter = $null
$filter = $null
$result = $null
$request = $null
$proxy = $null

}
