# example collect script to collect data from the collection service proxy

# execute this before the script so that the DTS proxy variable is set up:

# $proxy = New-WebServiceProxy -Uri 'http://localhost:5557/DTSPSP' -namespace DTS

# this can only be done once per powershell session since powershell has a bug
# that will create a superfluous casting error if the proxy is created twice

function collectex
{

# create object to collect
$object = New-Object DTS.PerformanceObject
$object.Name = 'Process'

#create the filter
$filter = New-Object DTS.PerformanceObjectInstanceFilter
$filter.ObjectName = 'Process'
$filter.InstanceNames = @('svchost','DmPerfss')

#counters
$countertestfilter = New-Object DTS.PerformanceCounterTestFilter
$countertestfilter.ObjectName = 'Process'
$countertestfilter.counterName = '% Processor Time'
$countertestfilter.ThresholdValue = 0.1
$countertestfilter.ThresholdValueSpecified = $true

$testtype = New-Object DTS.PerformanceCounterTestType
$testtype.value__ = 1  # enums are converted to int in ascending order starting with zero LT,GT,LTE,GTE,EQ
$countertestfilter.Type = $testtype
$countertestfilter.TypeSpecified = $true

# filter the counters
$counter = New-Object DTS.PerformanceCounter
$counter.Name = '% Processor Time'

$counter2 = New-Object DTS.PerformanceCounter
$counter2.Name = '% User Time'

$counter3 = New-Object DTS.PerformanceCounter
$counter3.Name = 'ID Process'

$object.counters = @($counter,$counter2,$counter3)

# set up the memory object to collect all of the counters
$mobject = New-Object DTS.PerformanceObject
$mobject.Name = 'Memory'

$mcounters = $proxy.GetCounterNames('Memory')
if ($mcounters -eq $null)
{
    Write-Host 'Could not get the counternames for the Memory object'  
}
else
{
    $mobject.counters = @()
    foreach ($countername in $mcounters)
    {
        $mcounter = New-Object DTS.PerformanceCounter
        $mcounter.Name = $countername;
        $mobject.counters += @($mcounter)
    }

    # set up the time
    $endtime = [System.DateTime]::Now
    $duration = New-Object System.TimeSpan(0,0,30,0)
    $starttime = $endtime - $duration

    # create the request
    $request = New-Object DTS.PerformanceDataRequest
    $request.ObjectsRequested = @($object,$mobject)
    $request.InstanceFilters = @($filter)
    $request.PerformanceCounterTestFilters = @($countertestfilter);
    $request.StartTime = $starttime
    $request.EndTime = $endtime
    $request.StartTimeSpecified = $true
    $request.EndTimeSpecified = $true

    Write-Host 'Collecting from ' $uri ' Object(s) ' $request.ObjectsRequested ' FilteredInstances(s) ' $request.InstanceFilters
    Write-Host 'From ' $starttime ' To ' $endtime

    # get the data
    $data = $proxy.GetPerformanceData($request)

    # Data is returned in the ObjectData array for each object name
    for ($i = 0; $i -lt $data.ObjectNames.Length; $i++)                
    {
        # samples of the object are stored in the ObjectData array sorted by Time
        for ($s = 0; $s -lt $data.ObjectData[$i].Length; $s++)
        {
            # Get the sample of the ith object
            $sample = $data.ObjectData[$i][$s]
                                    
            $message = $data.ObjectNames[$i] + ',' + $sample.CollectionTime
                   
            # Each sample has a number of instances, for each instance output the counter values
            for ($j=0; $j -lt $sample.InstanceNames.Length; $j++)        
            {
            
                $message =  $message + "`r`n" + $sample.InstanceNames[$j]

                # Each instance has a number of values ordered by the request counter order
                for ($k = 0; $k -lt $sample.Values[$j].Length; $k++)
                {                            
                    $message = $message + ','

                    if ($sample.Values[$j][$k] -ne $null)
                    {
                        $message = $message + $sample.Values[$j][$k]
                    }
                }
            }

           Write-Host $message
        }
    }
}

$uri = $null
$message = $null
$object = $null
$counter = $null
$filter = $null
$result = $null
$request = $null
$proxy = $null

}
